/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_CANSERVICE_CANSERVICE_PROXY_H
#define JAC_AP_CANSERVICE_CANSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/canservice/canservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace canservice {
namespace proxy {
namespace events {
    using INS_CAN2SWC_100HZ_Event = ara::com::internal::proxy::event::EventAdapter<::INS_CAN2SWC_100HZ>;
    using IMU_CAN2SWC_100HZ_Event = ara::com::internal::proxy::event::EventAdapter<::IMU_CAN2SWC_100HZ>;
    using GNSS_CAN2SWC_5HZ_Event = ara::com::internal::proxy::event::EventAdapter<::GNSS_CAN2SWC_5HZ>;
    using VEHICLE_CAN2SWC_50HZ_Event = ara::com::internal::proxy::event::EventAdapter<::VEHICLE_CAN2SWC_50HZ>;
    using VEHICLE_CAN2SWC_10HZ_Event = ara::com::internal::proxy::event::EventAdapter<::VEHICLE_CAN2SWC_10HZ>;
    using CAN2SWCEvent = ara::com::internal::proxy::event::EventAdapter<::CAN2SWC>;
    using VEHICLE_CAN_MCU2SOC_50HZ_Event = ara::com::internal::proxy::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_50HZ>;
    using VEHICLE_CAN_MCU2SOC_10HZ_Event = ara::com::internal::proxy::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_10HZ>;
    using VEHICLE_CAN_MCU2SOC_100HZ_Event = ara::com::internal::proxy::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_100HZ>;
    using VEHICLE_CAN_MCU2SOC_2HZ_Event = ara::com::internal::proxy::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_2HZ>;
    static constexpr ara::com::internal::EntityId CANServiceINS_CAN2SWC_100HZ_EventId = 6447U; //INS_CAN2SWC_100HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceIMU_CAN2SWC_100HZ_EventId = 21038U; //IMU_CAN2SWC_100HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceGNSS_CAN2SWC_5HZ_EventId = 21370U; //GNSS_CAN2SWC_5HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN2SWC_50HZ_EventId = 25430U; //VEHICLE_CAN2SWC_50HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN2SWC_10HZ_EventId = 17496U; //VEHICLE_CAN2SWC_10HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceCAN2SWCEventId = 15226U; //CAN2SWCEvent_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId = 16667U; //VEHICLE_CAN_MCU2SOC_50HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId = 10596U; //VEHICLE_CAN_MCU2SOC_10HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId = 64925U; //VEHICLE_CAN_MCU2SOC_100HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId = 26133U; //VEHICLE_CAN_MCU2SOC_2HZ_Event_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class CANServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              INS_CAN2SWC_100HZ_Event(proxyAdapter->GetProxy(), events::CANServiceINS_CAN2SWC_100HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              IMU_CAN2SWC_100HZ_Event(proxyAdapter->GetProxy(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              GNSS_CAN2SWC_5HZ_Event(proxyAdapter->GetProxy(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VEHICLE_CAN2SWC_50HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VEHICLE_CAN2SWC_10HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              CAN2SWCEvent(proxyAdapter->GetProxy(), events::CANServiceCAN2SWCEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VEHICLE_CAN_MCU2SOC_50HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VEHICLE_CAN_MCU2SOC_10HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VEHICLE_CAN_MCU2SOC_100HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VEHICLE_CAN_MCU2SOC_2HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::INS_CAN2SWC_100HZ_Event GetINS_CAN2SWC_100HZ_Event() noexcept { return std::move(INS_CAN2SWC_100HZ_Event); }
        events::IMU_CAN2SWC_100HZ_Event GetIMU_CAN2SWC_100HZ_Event() noexcept { return std::move(IMU_CAN2SWC_100HZ_Event); }
        events::GNSS_CAN2SWC_5HZ_Event GetGNSS_CAN2SWC_5HZ_Event() noexcept { return std::move(GNSS_CAN2SWC_5HZ_Event); }
        events::VEHICLE_CAN2SWC_50HZ_Event GetVEHICLE_CAN2SWC_50HZ_Event() noexcept { return std::move(VEHICLE_CAN2SWC_50HZ_Event); }
        events::VEHICLE_CAN2SWC_10HZ_Event GetVEHICLE_CAN2SWC_10HZ_Event() noexcept { return std::move(VEHICLE_CAN2SWC_10HZ_Event); }
        events::CAN2SWCEvent GetCAN2SWCEvent() noexcept { return std::move(CAN2SWCEvent); }
        events::VEHICLE_CAN_MCU2SOC_50HZ_Event GetVEHICLE_CAN_MCU2SOC_50HZ_Event() noexcept { return std::move(VEHICLE_CAN_MCU2SOC_50HZ_Event); }
        events::VEHICLE_CAN_MCU2SOC_10HZ_Event GetVEHICLE_CAN_MCU2SOC_10HZ_Event() noexcept { return std::move(VEHICLE_CAN_MCU2SOC_10HZ_Event); }
        events::VEHICLE_CAN_MCU2SOC_100HZ_Event GetVEHICLE_CAN_MCU2SOC_100HZ_Event() noexcept { return std::move(VEHICLE_CAN_MCU2SOC_100HZ_Event); }
        events::VEHICLE_CAN_MCU2SOC_2HZ_Event GetVEHICLE_CAN_MCU2SOC_2HZ_Event() noexcept { return std::move(VEHICLE_CAN_MCU2SOC_2HZ_Event); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::INS_CAN2SWC_100HZ_Event INS_CAN2SWC_100HZ_Event;
        events::IMU_CAN2SWC_100HZ_Event IMU_CAN2SWC_100HZ_Event;
        events::GNSS_CAN2SWC_5HZ_Event GNSS_CAN2SWC_5HZ_Event;
        events::VEHICLE_CAN2SWC_50HZ_Event VEHICLE_CAN2SWC_50HZ_Event;
        events::VEHICLE_CAN2SWC_10HZ_Event VEHICLE_CAN2SWC_10HZ_Event;
        events::CAN2SWCEvent CAN2SWCEvent;
        events::VEHICLE_CAN_MCU2SOC_50HZ_Event VEHICLE_CAN_MCU2SOC_50HZ_Event;
        events::VEHICLE_CAN_MCU2SOC_10HZ_Event VEHICLE_CAN_MCU2SOC_10HZ_Event;
        events::VEHICLE_CAN_MCU2SOC_100HZ_Event VEHICLE_CAN_MCU2SOC_100HZ_Event;
        events::VEHICLE_CAN_MCU2SOC_2HZ_Event VEHICLE_CAN_MCU2SOC_2HZ_Event;
    };

    virtual ~CANServiceProxy()
    {
        INS_CAN2SWC_100HZ_Event.UnsetReceiveHandler();
        INS_CAN2SWC_100HZ_Event.Unsubscribe();
        IMU_CAN2SWC_100HZ_Event.UnsetReceiveHandler();
        IMU_CAN2SWC_100HZ_Event.Unsubscribe();
        GNSS_CAN2SWC_5HZ_Event.UnsetReceiveHandler();
        GNSS_CAN2SWC_5HZ_Event.Unsubscribe();
        VEHICLE_CAN2SWC_50HZ_Event.UnsetReceiveHandler();
        VEHICLE_CAN2SWC_50HZ_Event.Unsubscribe();
        VEHICLE_CAN2SWC_10HZ_Event.UnsetReceiveHandler();
        VEHICLE_CAN2SWC_10HZ_Event.Unsubscribe();
        CAN2SWCEvent.UnsetReceiveHandler();
        CAN2SWCEvent.Unsubscribe();
        VEHICLE_CAN_MCU2SOC_50HZ_Event.UnsetReceiveHandler();
        VEHICLE_CAN_MCU2SOC_50HZ_Event.Unsubscribe();
        VEHICLE_CAN_MCU2SOC_10HZ_Event.UnsetReceiveHandler();
        VEHICLE_CAN_MCU2SOC_10HZ_Event.Unsubscribe();
        VEHICLE_CAN_MCU2SOC_100HZ_Event.UnsetReceiveHandler();
        VEHICLE_CAN_MCU2SOC_100HZ_Event.Unsubscribe();
        VEHICLE_CAN_MCU2SOC_2HZ_Event.UnsetReceiveHandler();
        VEHICLE_CAN_MCU2SOC_2HZ_Event.Unsubscribe();

    }

    explicit CANServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, handle)),
          INS_CAN2SWC_100HZ_Event(proxyAdapter->GetProxy(), events::CANServiceINS_CAN2SWC_100HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          IMU_CAN2SWC_100HZ_Event(proxyAdapter->GetProxy(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          GNSS_CAN2SWC_5HZ_Event(proxyAdapter->GetProxy(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VEHICLE_CAN2SWC_50HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VEHICLE_CAN2SWC_10HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          CAN2SWCEvent(proxyAdapter->GetProxy(), events::CANServiceCAN2SWCEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VEHICLE_CAN_MCU2SOC_50HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VEHICLE_CAN_MCU2SOC_10HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VEHICLE_CAN_MCU2SOC_100HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VEHICLE_CAN_MCU2SOC_2HZ_Event(proxyAdapter->GetProxy(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    CANServiceProxy(const CANServiceProxy&) = delete;
    CANServiceProxy& operator=(const CANServiceProxy&) = delete;

    CANServiceProxy(CANServiceProxy&&) = default;
    CANServiceProxy& operator=(CANServiceProxy&&) = default;
    CANServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          INS_CAN2SWC_100HZ_Event(token.GetINS_CAN2SWC_100HZ_Event()),
          IMU_CAN2SWC_100HZ_Event(token.GetIMU_CAN2SWC_100HZ_Event()),
          GNSS_CAN2SWC_5HZ_Event(token.GetGNSS_CAN2SWC_5HZ_Event()),
          VEHICLE_CAN2SWC_50HZ_Event(token.GetVEHICLE_CAN2SWC_50HZ_Event()),
          VEHICLE_CAN2SWC_10HZ_Event(token.GetVEHICLE_CAN2SWC_10HZ_Event()),
          CAN2SWCEvent(token.GetCAN2SWCEvent()),
          VEHICLE_CAN_MCU2SOC_50HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_50HZ_Event()),
          VEHICLE_CAN_MCU2SOC_10HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_10HZ_Event()),
          VEHICLE_CAN_MCU2SOC_100HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_100HZ_Event()),
          VEHICLE_CAN_MCU2SOC_2HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_2HZ_Event()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::canservice::CANService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<CANServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::canservice::CANService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::canservice::CANService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::canservice::CANService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::canservice::CANService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::INS_CAN2SWC_100HZ_Event INS_CAN2SWC_100HZ_Event;
    events::IMU_CAN2SWC_100HZ_Event IMU_CAN2SWC_100HZ_Event;
    events::GNSS_CAN2SWC_5HZ_Event GNSS_CAN2SWC_5HZ_Event;
    events::VEHICLE_CAN2SWC_50HZ_Event VEHICLE_CAN2SWC_50HZ_Event;
    events::VEHICLE_CAN2SWC_10HZ_Event VEHICLE_CAN2SWC_10HZ_Event;
    events::CAN2SWCEvent CAN2SWCEvent;
    events::VEHICLE_CAN_MCU2SOC_50HZ_Event VEHICLE_CAN_MCU2SOC_50HZ_Event;
    events::VEHICLE_CAN_MCU2SOC_10HZ_Event VEHICLE_CAN_MCU2SOC_10HZ_Event;
    events::VEHICLE_CAN_MCU2SOC_100HZ_Event VEHICLE_CAN_MCU2SOC_100HZ_Event;
    events::VEHICLE_CAN_MCU2SOC_2HZ_Event VEHICLE_CAN_MCU2SOC_2HZ_Event;
};
} // namespace proxy
} // namespace canservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_CANSERVICE_CANSERVICE_PROXY_H
